import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestNSAccessibilityHelper(AppKit.NSObject):
    def accessibilityIsAttributeSettable_(self, arg):
        return 1

    def accessibilityIsIgnored(self):
        return 1

    def accessibilityHitTest_(self, pt):
        pass

    def accessibilitySetOverrideValue_forAttribute_(self, v, a):
        return 1

    def accessibilityNotifiesWhenDestroyed(self):
        pass

    def accessibilityIndexOfChild_(self, c):
        return 1

    def accessibilityArrayAttributeCount_(self, c):
        return 1

    def accessibilityArrayAttributeValues_index_maxCount_(self, v, i, c):
        return None


class TestNSAccessibility(TestCase):
    def testInformal(self):
        self.assertResultIsBOOL(
            TestNSAccessibilityHelper.accessibilityIsAttributeSettable_
        )
        self.assertResultIsBOOL(TestNSAccessibilityHelper.accessibilityIsIgnored)
        self.assertArgHasType(
            TestNSAccessibilityHelper.accessibilityHitTest_,
            0,
            AppKit.NSPoint.__typestr__,
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityHelper.accessibilitySetOverrideValue_forAttribute_
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityHelper.accessibilityNotifiesWhenDestroyed
        )
        self.assertResultHasType(
            TestNSAccessibilityHelper.accessibilityIndexOfChild_, objc._C_NSUInteger
        )
        self.assertResultHasType(
            TestNSAccessibilityHelper.accessibilityArrayAttributeCount_,
            objc._C_NSUInteger,
        )
        self.assertResultHasType(
            TestNSAccessibilityHelper.accessibilityArrayAttributeCount_,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSAccessibilityHelper.accessibilityArrayAttributeValues_index_maxCount_,
            1,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSAccessibilityHelper.accessibilityArrayAttributeValues_index_maxCount_,
            2,
            objc._C_NSUInteger,
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(
            AppKit.NSWorkspace.accessibilityDisplayShouldIncreaseContrast
        )
        self.assertResultIsBOOL(
            AppKit.NSWorkspace.accessibilityDisplayShouldDifferentiateWithoutColor
        )
        self.assertResultIsBOOL(
            AppKit.NSWorkspace.accessibilityDisplayShouldReduceTransparency
        )

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(
            AppKit.NSWorkspace.accessibilityDisplayShouldReduceMotion
        )
        self.assertResultIsBOOL(
            AppKit.NSWorkspace.accessibilityDisplayShouldInvertColors
        )

    @min_os_level("10.10")
    def testFunctions10_10(self):
        AppKit.NSAccessibilityFrameInView  # Existence
        AppKit.NSAccessibilityPointInView  # Existence

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(
            AppKit.NSWorkspaceAccessibilityDisplayOptionsDidChangeNotification, str
        )
        self.assertIsInstance(AppKit.NSAccessibilityActivationPointAttribute, str)

        self.assertIsInstance(AppKit.NSAccessibilitySharedFocusElementsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityAlternateUIVisibleAttribute, str)

        self.assertEqual(AppKit.NSAccessibilityOrientationUnknown, 0)
        self.assertEqual(AppKit.NSAccessibilityOrientationVertical, 1)
        self.assertEqual(AppKit.NSAccessibilityOrientationHorizontal, 2)

        self.assertEqual(AppKit.NSAccessibilitySortDirectionUnknown, 0)
        self.assertEqual(AppKit.NSAccessibilitySortDirectionAscending, 1)
        self.assertEqual(AppKit.NSAccessibilitySortDirectionDescending, 2)

        self.assertEqual(AppKit.NSAccessibilityRulerMarkerTypeUnknown, 0)
        self.assertEqual(AppKit.NSAccessibilityRulerMarkerTypeTabStopLeft, 1)
        self.assertEqual(AppKit.NSAccessibilityRulerMarkerTypeTabStopRight, 2)
        self.assertEqual(AppKit.NSAccessibilityRulerMarkerTypeTabStopCenter, 3)
        self.assertEqual(AppKit.NSAccessibilityRulerMarkerTypeTabStopDecimal, 4)
        self.assertEqual(AppKit.NSAccessibilityRulerMarkerTypeIndentHead, 5)
        self.assertEqual(AppKit.NSAccessibilityRulerMarkerTypeIndentTail, 6)
        self.assertEqual(AppKit.NSAccessibilityRulerMarkerTypeIndentFirstLine, 7)

        self.assertEqual(AppKit.NSAccessibilityUnitsUnknown, 0)
        self.assertEqual(AppKit.NSAccessibilityUnitsInches, 1)
        self.assertEqual(AppKit.NSAccessibilityUnitsCentimeters, 2)
        self.assertEqual(AppKit.NSAccessibilityUnitsPoints, 3)
        self.assertEqual(AppKit.NSAccessibilityUnitsPicas, 4)

    def testFunction(self):
        v = AppKit.NSAccessibilityRoleDescription(
            AppKit.NSAccessibilityButtonRole, None
        )
        self.assertIsInstance(v, str)

        b = AppKit.NSButton.alloc().init()
        v = AppKit.NSAccessibilityRoleDescriptionForUIElement(b)
        self.assertIsInstance(v, str)

        v = AppKit.NSAccessibilityActionDescription(
            AppKit.NSAccessibilityIncrementAction
        )
        self.assertIsInstance(v, str)

        self.assertRaises(
            objc.error,
            AppKit.NSAccessibilityRaiseBadArgumentException,
            b,
            "attribute",
            "value",
        )

        v = AppKit.NSAccessibilityUnignoredAncestor(b)
        self.assertIs(v, None)
        v = AppKit.NSAccessibilityUnignoredDescendant(b)
        self.assertIsInstance(b, AppKit.NSView)

        v = AppKit.NSAccessibilityUnignoredChildren([b])
        self.assertIsInstance(v, AppKit.NSArray)

        v = AppKit.NSAccessibilityUnignoredChildrenForOnlyChild(b)
        self.assertIsInstance(v, AppKit.NSArray)

        v = AppKit.NSAccessibilityPostNotification(b, "hello")
        self.assertIs(v, None)

    def testConstants(self):
        self.assertEqual(AppKit.NSAccessibilityAnnotationPositionFullRange, 0)
        self.assertEqual(AppKit.NSAccessibilityAnnotationPositionStart, 1)
        self.assertEqual(AppKit.NSAccessibilityAnnotationPositionEnd, 2)

        self.assertIsInstance(AppKit.NSAccessibilityErrorCodeExceptionInfo, str)
        self.assertIsInstance(AppKit.NSAccessibilityRoleAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityRoleDescriptionAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilitySubroleAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityHelpAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityValueAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityMinValueAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityMaxValueAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityEnabledAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityFocusedAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityParentAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityChildrenAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityWindowAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityTopLevelUIElementAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilitySelectedChildrenAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityVisibleChildrenAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityPositionAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilitySizeAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityContentsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityTitleAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityDescriptionAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityShownMenuAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityValueDescriptionAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityPreviousContentsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityNextContentsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityHeaderAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityEditedAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityTabsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityHorizontalScrollBarAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityVerticalScrollBarAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityOverflowButtonAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityIncrementButtonAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityDecrementButtonAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityFilenameAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityExpandedAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilitySelectedAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilitySplittersAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityDocumentAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityURLAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityIndexAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityRowCountAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityColumnCountAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityOrderedByRowAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityTitleUIElementAttribute, str)
        self.assertIsInstance(
            AppKit.NSAccessibilityServesAsTitleForUIElementsAttribute, str
        )
        self.assertIsInstance(AppKit.NSAccessibilityLinkedUIElementsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilitySelectedTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilitySelectedTextRangeAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityNumberOfCharactersAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityVisibleCharacterRangeAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilitySharedTextUIElementsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilitySharedCharacterRangeAttribute, str)
        self.assertIsInstance(
            AppKit.NSAccessibilityInsertionPointLineNumberAttribute, str
        )
        self.assertIsInstance(AppKit.NSAccessibilitySelectedTextRangesAttribute, str)
        self.assertIsInstance(
            AppKit.NSAccessibilityLineForIndexParameterizedAttribute, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilityRangeForLineParameterizedAttribute, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilityStringForRangeParameterizedAttribute, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilityRangeForPositionParameterizedAttribute, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilityRangeForIndexParameterizedAttribute, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilityBoundsForRangeParameterizedAttribute, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilityRTFForRangeParameterizedAttribute, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilityStyleRangeForIndexParameterizedAttribute, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilityAttributedStringForRangeParameterizedAttribute, str
        )
        self.assertIsInstance(AppKit.NSAccessibilityFontTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityForegroundColorTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityBackgroundColorTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityUnderlineColorTextAttribute, str)
        self.assertIsInstance(
            AppKit.NSAccessibilityStrikethroughColorTextAttribute, str
        )
        self.assertIsInstance(AppKit.NSAccessibilityUnderlineTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilitySuperscriptTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityStrikethroughTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityShadowTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityAttachmentTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityLinkTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityMisspelledTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityMarkedMisspelledTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityFontNameKey, str)
        self.assertIsInstance(AppKit.NSAccessibilityFontFamilyKey, str)
        self.assertIsInstance(AppKit.NSAccessibilityVisibleNameKey, str)
        self.assertIsInstance(AppKit.NSAccessibilityFontSizeKey, str)
        self.assertIsInstance(AppKit.NSAccessibilityMainAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityMinimizedAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityCloseButtonAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityZoomButtonAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityMinimizeButtonAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityToolbarButtonAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityProxyAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityGrowAreaAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityModalAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityDefaultButtonAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityCancelButtonAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityMenuBarAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityWindowsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityFrontmostAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityHiddenAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityMainWindowAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityFocusedWindowAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityFocusedUIElementAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityOrientationAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityVerticalOrientationValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityHorizontalOrientationValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityColumnTitlesAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilitySearchButtonAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilitySearchMenuAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityClearButtonAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityRowsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityVisibleRowsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilitySelectedRowsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityColumnsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityVisibleColumnsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilitySelectedColumnsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilitySortDirectionAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityAscendingSortDirectionValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityDescendingSortDirectionValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityUnknownSortDirectionValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityDisclosingAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityDisclosedRowsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityDisclosedByRowAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityDisclosureLevelAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityAllowedValuesAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityLabelUIElementsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityLabelValueAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityMatteHoleAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityMatteContentUIElementAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityMarkerUIElementsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityMarkerValuesAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityMarkerGroupUIElementAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityUnitsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityUnitDescriptionAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityMarkerTypeAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityMarkerTypeDescriptionAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityLeftTabStopMarkerTypeValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityRightTabStopMarkerTypeValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityCenterTabStopMarkerTypeValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityDecimalTabStopMarkerTypeValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityHeadIndentMarkerTypeValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityTailIndentMarkerTypeValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityFirstLineIndentMarkerTypeValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityUnknownMarkerTypeValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityInchesUnitValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityCentimetersUnitValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityPointsUnitValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityPicasUnitValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityUnknownUnitValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityPressAction, str)
        self.assertIsInstance(AppKit.NSAccessibilityIncrementAction, str)
        self.assertIsInstance(AppKit.NSAccessibilityDecrementAction, str)
        self.assertIsInstance(AppKit.NSAccessibilityConfirmAction, str)
        self.assertIsInstance(AppKit.NSAccessibilityPickAction, str)
        self.assertIsInstance(AppKit.NSAccessibilityCancelAction, str)
        self.assertIsInstance(AppKit.NSAccessibilityRaiseAction, str)
        self.assertIsInstance(AppKit.NSAccessibilityShowMenuAction, str)
        self.assertIsInstance(AppKit.NSAccessibilityDeleteAction, str)
        self.assertIsInstance(AppKit.NSAccessibilityMainWindowChangedNotification, str)
        self.assertIsInstance(
            AppKit.NSAccessibilityFocusedWindowChangedNotification, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilityFocusedUIElementChangedNotification, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilityApplicationActivatedNotification, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilityApplicationDeactivatedNotification, str
        )
        self.assertIsInstance(AppKit.NSAccessibilityApplicationHiddenNotification, str)
        self.assertIsInstance(AppKit.NSAccessibilityApplicationShownNotification, str)
        self.assertIsInstance(AppKit.NSAccessibilityWindowCreatedNotification, str)
        self.assertIsInstance(AppKit.NSAccessibilityWindowMovedNotification, str)
        self.assertIsInstance(AppKit.NSAccessibilityWindowResizedNotification, str)
        self.assertIsInstance(AppKit.NSAccessibilityWindowMiniaturizedNotification, str)
        self.assertIsInstance(
            AppKit.NSAccessibilityWindowDeminiaturizedNotification, str
        )
        self.assertIsInstance(AppKit.NSAccessibilityDrawerCreatedNotification, str)
        self.assertIsInstance(AppKit.NSAccessibilitySheetCreatedNotification, str)
        self.assertIsInstance(AppKit.NSAccessibilityUIElementDestroyedNotification, str)
        self.assertIsInstance(AppKit.NSAccessibilityValueChangedNotification, str)
        self.assertIsInstance(AppKit.NSAccessibilityTitleChangedNotification, str)
        self.assertIsInstance(AppKit.NSAccessibilityResizedNotification, str)
        self.assertIsInstance(AppKit.NSAccessibilityMovedNotification, str)
        self.assertIsInstance(AppKit.NSAccessibilityCreatedNotification, str)
        self.assertIsInstance(AppKit.NSAccessibilityHelpTagCreatedNotification, str)
        self.assertIsInstance(
            AppKit.NSAccessibilitySelectedTextChangedNotification, str
        )
        self.assertIsInstance(AppKit.NSAccessibilityRowCountChangedNotification, str)
        self.assertIsInstance(
            AppKit.NSAccessibilitySelectedChildrenChangedNotification, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilitySelectedRowsChangedNotification, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilitySelectedColumnsChangedNotification, str
        )
        self.assertIsInstance(AppKit.NSAccessibilityUnknownRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityButtonRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityRadioButtonRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityCheckBoxRole, str)
        self.assertIsInstance(AppKit.NSAccessibilitySliderRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityTabGroupRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityTextFieldRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityStaticTextRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityTextAreaRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityScrollAreaRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityPopUpButtonRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityMenuButtonRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityTableRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityApplicationRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityGroupRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityRadioGroupRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityListRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityScrollBarRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityValueIndicatorRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityImageRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityMenuBarRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityMenuRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityMenuItemRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityColumnRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityRowRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityToolbarRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityBusyIndicatorRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityProgressIndicatorRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityWindowRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityDrawerRole, str)
        self.assertIsInstance(AppKit.NSAccessibilitySystemWideRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityOutlineRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityIncrementorRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityBrowserRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityComboBoxRole, str)
        self.assertIsInstance(AppKit.NSAccessibilitySplitGroupRole, str)
        self.assertIsInstance(AppKit.NSAccessibilitySplitterRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityColorWellRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityGrowAreaRole, str)
        self.assertIsInstance(AppKit.NSAccessibilitySheetRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityHelpTagRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityMatteRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityRulerRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityRulerMarkerRole, str)
        self.assertIsInstance(AppKit.NSAccessibilitySortButtonRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityLinkRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityDisclosureTriangleRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityGridRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityUnknownSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityCloseButtonSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityZoomButtonSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityMinimizeButtonSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityToolbarButtonSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityTableRowSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityOutlineRowSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilitySecureTextFieldSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityStandardWindowSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityDialogSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilitySystemDialogSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityFloatingWindowSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilitySystemFloatingWindowSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityIncrementArrowSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityDecrementArrowSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityIncrementPageSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityDecrementPageSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilitySearchFieldSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityTextAttachmentSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityTextLinkSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityTimelineSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityRelevanceIndicatorRole, str)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(AppKit.NSAccessibilityUnknownOrientationValue, str)
        self.assertIsInstance(AppKit.NSAccessibilityWarningValueAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityCriticalValueAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityPlaceholderValueAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilitySelectedCellsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityVisibleCellsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityRowHeaderUIElementsAttribute, str)
        self.assertIsInstance(
            AppKit.NSAccessibilityColumnHeaderUIElementsAttribute, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilityCellForColumnAndRowParameterizedAttribute, str
        )
        self.assertIsInstance(AppKit.NSAccessibilityRowIndexRangeAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityColumnIndexRangeAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityHorizontalUnitsAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityVerticalUnitsAttribute, str)
        self.assertIsInstance(
            AppKit.NSAccessibilityHorizontalUnitDescriptionAttribute, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilityVerticalUnitDescriptionAttribute, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilityLayoutPointForScreenPointParameterizedAttribute, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilityLayoutSizeForScreenSizeParameterizedAttribute, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilityScreenPointForLayoutPointParameterizedAttribute, str
        )
        self.assertIsInstance(
            AppKit.NSAccessibilityScreenSizeForLayoutSizeParameterizedAttribute, str
        )
        self.assertIsInstance(AppKit.NSAccessibilityHandlesAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityRowExpandedNotification, str)
        self.assertIsInstance(AppKit.NSAccessibilityRowCollapsedNotification, str)
        self.assertIsInstance(
            AppKit.NSAccessibilitySelectedCellsChangedNotification, str
        )
        self.assertIsInstance(AppKit.NSAccessibilityUnitsChangedNotification, str)
        self.assertIsInstance(
            AppKit.NSAccessibilitySelectedChildrenMovedNotification, str
        )
        self.assertIsInstance(AppKit.NSAccessibilitySortButtonRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityLevelIndicatorRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityCellRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityLayoutAreaRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityLayoutItemRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityHandleRole, str)
        self.assertIsInstance(AppKit.NSAccessibilitySortButtonSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityRatingIndicatorSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityContentListSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityDefinitionListSubrole, str)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(AppKit.NSAccessibilityAutocorrectedTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityFullScreenButtonAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityPopoverRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityFullScreenButtonSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityIdentifierAttribute, str)
        self.assertIsInstance(
            AppKit.NSAccessibilityAnnouncementRequestedNotification, str
        )
        self.assertIsInstance(AppKit.NSAccessibilityAnnouncementKey, str)

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertIsInstance(AppKit.NSAccessibilityExtrasMenuBarAttribute, str)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertIsInstance(
            AppKit.NSAccessibilityContainsProtectedContentAttribute, str
        )
        self.assertIsInstance(AppKit.NSAccessibilityShowAlternateUIAction, str)
        self.assertIsInstance(AppKit.NSAccessibilityShowDefaultUIAction, str)
        self.assertIsInstance(AppKit.NSAccessibilityLayoutChangedNotification, str)
        self.assertIsInstance(AppKit.NSAccessibilityToggleSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilitySwitchSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityDescriptionListSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityUIElementsKey, str)
        self.assertIsInstance(AppKit.NSAccessibilityPriorityKey, str)

        self.assertEqual(AppKit.NSAccessibilityPriorityLow, 10)
        self.assertEqual(AppKit.NSAccessibilityPriorityMedium, 50)
        self.assertEqual(AppKit.NSAccessibilityPriorityHigh, 90)

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertIsInstance(AppKit.NSAccessibilityListItemPrefixTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityListItemIndexTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityListItemLevelTextAttribute, str)

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertIsInstance(AppKit.NSAccessibilityRequiredAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityMenuBarItemRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityTextAlignmentAttribute, str)

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(AppKit.NSAccessibilityAnnotationLabel, str)
        self.assertIsInstance(AppKit.NSAccessibilityAnnotationElement, str)
        self.assertIsInstance(AppKit.NSAccessibilityAnnotationLocation, str)

        self.assertIsInstance(AppKit.NSAccessibilityLanguageTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityCustomTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityAnnotationTextAttribute, str)
        self.assertIsInstance(AppKit.NSAccessibilityPageRole, str)
        self.assertIsInstance(AppKit.NSAccessibilityTabButtonSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilityCollectionListSubrole, str)
        self.assertIsInstance(AppKit.NSAccessibilitySectionListSubrole, str)

    @min_os_level("10.7")
    def testFunctions10_7(self):
        self.assertIsInstance(
            AppKit.NSAccessibilityPostNotificationWithUserInfo, objc.function
        )

    @min_os_level("10.9")
    def testFunctions10_9(self):
        self.assertArgIsBOOL(AppKit.NSAccessibilitySetMayContainProtectedContent, 0)
        self.assertResultIsBOOL(AppKit.NSAccessibilitySetMayContainProtectedContent)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSByteCountFormatter(TestCase):
    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertEqual(Foundation.NSByteCountFormatterUseDefault, 0)
        self.assertEqual(Foundation.NSByteCountFormatterUseBytes, 1 << 0)
        self.assertEqual(Foundation.NSByteCountFormatterUseKB, 1 << 1)
        self.assertEqual(Foundation.NSByteCountFormatterUseMB, 1 << 2)
        self.assertEqual(Foundation.NSByteCountFormatterUseGB, 1 << 3)
        self.assertEqual(Foundation.NSByteCountFormatterUseTB, 1 << 4)
        self.assertEqual(Foundation.NSByteCountFormatterUsePB, 1 << 5)
        self.assertEqual(Foundation.NSByteCountFormatterUseEB, 1 << 6)
        self.assertEqual(Foundation.NSByteCountFormatterUseZB, 1 << 7)
        self.assertEqual(Foundation.NSByteCountFormatterUseYBOrHigher, 0x0FF << 8)
        self.assertEqual(Foundation.NSByteCountFormatterUseAll, 0x0FFFF)
        self.assertEqual(Foundation.NSByteCountFormatterCountStyleFile, 0)
        self.assertEqual(Foundation.NSByteCountFormatterCountStyleMemory, 1)
        self.assertEqual(Foundation.NSByteCountFormatterCountStyleDecimal, 2)
        self.assertEqual(Foundation.NSByteCountFormatterCountStyleBinary, 3)

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertResultIsBOOL(
            Foundation.NSByteCountFormatter.allowsNonnumericFormatting
        )
        self.assertArgIsBOOL(
            Foundation.NSByteCountFormatter.setAllowsNonnumericFormatting_, 0
        )

        self.assertResultIsBOOL(Foundation.NSByteCountFormatter.includesUnit)
        self.assertArgIsBOOL(Foundation.NSByteCountFormatter.setIncludesUnit_, 0)

        self.assertResultIsBOOL(Foundation.NSByteCountFormatter.includesCount)
        self.assertArgIsBOOL(Foundation.NSByteCountFormatter.setIncludesCount_, 0)

        self.assertResultIsBOOL(Foundation.NSByteCountFormatter.includesActualByteCount)
        self.assertArgIsBOOL(
            Foundation.NSByteCountFormatter.setIncludesActualByteCount_, 0
        )

        self.assertResultIsBOOL(Foundation.NSByteCountFormatter.isAdaptive)
        self.assertArgIsBOOL(Foundation.NSByteCountFormatter.setAdaptive_, 0)

        self.assertResultIsBOOL(Foundation.NSByteCountFormatter.zeroPadsFractionDigits)
        self.assertArgIsBOOL(
            Foundation.NSByteCountFormatter.setZeroPadsFractionDigits_, 0
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSFont(TestCase):
    def matrixEquals(self, value1, value2):
        self.assertEqual(len(value1), len(value2))
        for v1, v2 in zip(value1, value2):
            # This should probably be 'assertAlmostEquals'
            self.assertEqual(v1, v2)

    def testMatrixMethods(self):
        o = AppKit.NSFont.boldSystemFontOfSize_(10)
        m = o.matrix()
        self.assertTrue(isinstance(m, tuple))
        self.assertEqual(len(m), 6)

        nm = o.fontName()

        o = AppKit.NSFont.fontWithName_matrix_(nm, AppKit.NSFontIdentityMatrix)
        self.assertTrue(o is not None)

        m = o.matrix()
        self.assertIsInstance(m, tuple)
        self.assertEqual(len(m), 6)

        self.matrixEquals(m, (1.0, 0.0, 0.0, 1.0, 0.0, 0.0))

        # For some reason Tiger transforms this matrix to the one below. The
        # same thing happens in pure ObjC code.
        # o = AppKit.NSFont.fontWithName_matrix_(nm, (1.0, 2.0, 3.0, 4.0, 5.0, 6.0))
        o = AppKit.NSFont.fontWithName_matrix_(nm, (12.0, 0.0, 0.0, 12.0, 0.0, 0.0))
        self.assertTrue(o is not None)

        m = o.matrix()
        self.assertIsInstance(m, tuple)
        self.assertEqual(len(m), 6)

        # self.matrixEquals(m, (1.0, 2.0, 3.0, 4.0, 5.0, 6.0))
        self.matrixEquals(m, (12.0, 0.0, 0.0, 12.0, 0.0, 0.0))

        self.assertRaises(ValueError, AppKit.NSFont.fontWithName_matrix_, nm, "foo")
        self.assertRaises(
            ValueError, AppKit.NSFont.fontWithName_matrix_, nm, (1, 2, 3, 4)
        )
        self.assertRaises(
            ValueError,
            AppKit.NSFont.fontWithName_matrix_,
            nm,
            (1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0),
        )

    def testConstants(self):
        self.assertEqual(AppKit.NSFontIdentityMatrix, None)

        self.assertEqual(AppKit.NSControlGlyph, 0xFFFFFF)
        self.assertEqual(AppKit.NSNullGlyph, 0)
        self.assertEqual(AppKit.NSNativeShortGlyphPacking, 5)

        self.assertEqual(AppKit.NSFontDefaultRenderingMode, 0)
        self.assertEqual(AppKit.NSFontAntialiasedRenderingMode, 1)
        self.assertEqual(AppKit.NSFontIntegerAdvancementsRenderingMode, 2)
        self.assertEqual(AppKit.NSFontAntialiasedIntegerAdvancementsRenderingMode, 3)

        self.assertIsInstance(AppKit.NSAntialiasThresholdChangedNotification, str)
        self.assertIsInstance(AppKit.NSFontSetChangedNotification, str)

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsOut(AppKit.NSFont.getBoundingRects_forCGGlyphs_count_, 0)
        self.assertArgIsIn(AppKit.NSFont.getBoundingRects_forCGGlyphs_count_, 1)
        self.assertArgSizeInArg(AppKit.NSFont.getBoundingRects_forCGGlyphs_count_, 0, 2)
        self.assertArgSizeInArg(AppKit.NSFont.getBoundingRects_forCGGlyphs_count_, 1, 2)

        self.assertArgIsOut(AppKit.NSFont.getAdvancements_forCGGlyphs_count_, 0)
        self.assertArgIsIn(AppKit.NSFont.getAdvancements_forCGGlyphs_count_, 1)
        self.assertArgSizeInArg(AppKit.NSFont.getAdvancements_forCGGlyphs_count_, 0, 2)
        self.assertArgSizeInArg(AppKit.NSFont.getAdvancements_forCGGlyphs_count_, 1, 2)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(AppKit.NSFont.isVertical)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSFont.isFixedPitch)
        self.assertArgIsOut(AppKit.NSFont.getBoundingRects_forGlyphs_count_, 0)
        self.assertArgHasType(
            AppKit.NSFont.getBoundingRects_forGlyphs_count_, 1, b"n^I"
        )
        self.assertArgSizeInArg(AppKit.NSFont.getBoundingRects_forGlyphs_count_, 0, 2)
        self.assertArgSizeInArg(AppKit.NSFont.getBoundingRects_forGlyphs_count_, 1, 2)
        self.assertArgIsOut(AppKit.NSFont.getAdvancements_forGlyphs_count_, 0)
        self.assertArgHasType(AppKit.NSFont.getAdvancements_forGlyphs_count_, 1, b"n^I")
        self.assertArgSizeInArg(AppKit.NSFont.getAdvancements_forGlyphs_count_, 0, 2)
        self.assertArgSizeInArg(AppKit.NSFont.getAdvancements_forGlyphs_count_, 1, 2)
        if hasattr(AppKit.NSFont, "getAdvancements_forPackedGlyphs_count_"):
            self.assertArgIsOut(AppKit.NSFont.getAdvancements_forPackedGlyphs_count_, 0)
            self.assertArgIsIn(AppKit.NSFont.getAdvancements_forPackedGlyphs_count_, 1)
        self.assertArgSizeInArg(
            AppKit.NSFont.getAdvancements_forPackedGlyphs_length_, 0, 2
        )
        self.assertArgSizeInArg(
            AppKit.NSFont.getAdvancements_forPackedGlyphs_length_, 1, 2
        )

    def testFunctions(self):
        glyphs = [ord("A"), ord("B"), ord("9"), ord("a")]

        rv, packed = AppKit.NSConvertGlyphsToPackedGlyphs(
            glyphs, len(glyphs), AppKit.NSNativeShortGlyphPacking, None
        )
        self.assertIsInstance(rv, int)
        self.assertIsInstance(packed, str)
        if rv == 0:
            self.assertEqual(len(packed), 0)

        else:
            self.assertEqual(len(packed), rv)



import Cocoa
from MyBaseGradientView import MyBaseGradientView


class MyRectGradientView(MyBaseGradientView):
    def init(self):
        self = super().init()
        if self is None:
            return self

        self.myOffsetPt = Cocoa.NSMakePoint(0.0, 0.0)
        return self

    def drawRect_(self, rect):
        self.resetGradient()

        # if the "Radial Gradient" checkbox is turned on, draw using 'myOffsetPt'
        if self.myIsRadial:
            self.myGradient.drawInRect_relativeCenterPosition_(
                self.bounds(), self.myOffsetPt
            )

        else:
            self.myGradient.drawInRect_angle_(self.bounds(), self.myAngle)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSGlyphInfo(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSIdentityMappingCharacterCollection, 0)
        self.assertEqual(AppKit.NSAdobeCNS1CharacterCollection, 1)
        self.assertEqual(AppKit.NSAdobeGB1CharacterCollection, 2)
        self.assertEqual(AppKit.NSAdobeJapan1CharacterCollection, 3)
        self.assertEqual(AppKit.NSAdobeJapan2CharacterCollection, 4)
        self.assertEqual(AppKit.NSAdobeKorea1CharacterCollection, 5)



import traceback
import urllib
import urllib2

import Cocoa
import objc
from PyObjCTools import AppHelper


def serviceSelector(fn):
    # this is the signature of service selectors
    return objc.selector(fn, signature=b"v@:@@o^@")


def ERROR(s):
    # NSLog("ERROR: %s", s)
    return s


NAME = "TinyURLService-0.0"
TINYURL_API = "http://tinyurl.com/api-create.php"


def getTinyURL(url):
    data = urllib.urlencode({"url": url, "source": NAME})
    return urllib2.urlopen(TINYURL_API, data).read().decode("utf-8")


class TinyURLService(Cocoa.NSObject):
    @serviceSelector
    def doTinyURLService_userData_error_(self, pboard, data, error):
        # Mail.app in 10.4.1 doesn't do NSURLPboardType correctly!
        # Probably elsewhere too, so we just use strings.
        try:
            types = pboard.types()
            url = None

            if Cocoa.NSStringPboardType in types:
                urlString = pboard.stringForType_(Cocoa.NSStringPboardType)
                url = Cocoa.NSURL.URLWithString_(urlString.strip())
                if url is None:
                    return ERROR(
                        Cocoa.NSLocalizedString(
                            "Error: Given URL was not well-formed.",
                            "Given URL not well-formed.",
                        )
                    )

            if url is None:
                return ERROR(
                    Cocoa.NSLocalizedString(
                        "Error: Pasteboard doesn't contain a valid URL.",
                        "Pasteboard doesn't contain a valid URL.",
                    )
                )

            urlString = url.absoluteString()

            res = getTinyURL(urlString.UTF8String())

            resURL = Cocoa.NSURL.URLWithString_(res)
            if resURL is None:
                return ERROR(
                    Cocoa.NSLocalizedString(
                        "Error: Resultant URL was not well-formed.",
                        "Resultant URL not well-formed.",
                    )
                )
            pboard.declareTypes_owner_([Cocoa.NSStringPboardType], None)
            pboard.setString_forType_(resURL.absoluteString(), Cocoa.NSStringPboardType)
            return ERROR(None)
        except:  # noqa: E722, B001
            traceback.print_exc()
            return ERROR("Exception, see traceback")


def main():
    serviceProvider = TinyURLService.alloc().init()
    Cocoa.NSRegisterServicesProvider(serviceProvider, "TinyURLService")
    AppHelper.runConsoleEventLoop()


if __name__ == "__main__":
    main()



#
#  DNDTableView.py
#  Bookmarks
#
#  Converted by u.fiedler on 10.02.05.
#
#  The original version was written in Objective-C by Malcolm Crawford
#  at http://homepage.mac.com/mmalc/CocoaExamples/controllers.html

from AppKit import NSDragOperationLink, NSTableView
from objc import super


class DNDTableView(NSTableView):
    def draggingSourceOperationMaskForLocal_(self, flag):
        # This is a bug fix. See
        # file:///Developer/ADC%20Reference%20Library/documentation/Cocoa/Conceptual/DragandDrop/Tasks/faq.html#//apple_ref/doc/uid/20002248/BBCGGBHE  # noqa: B950
        # or http://developer.apple.com/documentation/Cocoa/Conceptual/DragandDrop/Tasks/faq.html#//apple_ref/doc/uid/20002248/BBCFIJGF  # noqa: B950
        if not flag:
            return NSDragOperationLink  # link for external dragged URLs
        return super().draggingSourceOperationMaskForLocal_(flag)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSCache(TestCase):
    @min_os_level("10.6")
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSCache.evictsObjectsWithDiscardedContent)
        self.assertArgIsBOOL(
            Foundation.NSCache.setEvictsObjectsWithDiscardedContent_, 0
        )

    @min_sdk_level("10.10")
    def testProtocols(self):
        objc.protocolNamed("NSCacheDelegate")

    def testConvenience(self):
        c = Foundation.NSCache.alloc().init()
        c.setObject_forKey_(42, "key")
        self.assertEqual(c.objectForKey_("key"), 42)
        self.assertEqual(c.objectForKey_("key"), 42)

        self.assertEqual(c["key"], 42)
        c["key"] = 21
        self.assertEqual(c["key"], 21)
        self.assertEqual(c.objectForKey_("key"), 21)

        self.assertEqual(c.get("key"), 21)
        self.assertEqual(c.get("key2"), None)

        c.clear()
        self.assertEqual(c.get("key"), None)

        with self.assertRaises(KeyError):
            c["key"]

        c["key"] = 1
        c["key2"] = 2

        self.assertEqual(c["key"], 1)
        self.assertEqual(c["key2"], 2)

        del c["key"]
        with self.assertRaises(KeyError):
            c["key"]
        self.assertEqual(c["key2"], 2)

        c["key"] = None
        self.assertEqual(c["key"], None)
        self.assertEqual(c.get("key"), None)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestOpenPanel(TestCase):
    def dont_testOpenPanelSignature(self):
        """
        This test failed sometime after the 1.0b1 release (on Panther).
        """

        o = AppKit.NSOpenPanel.openPanel()
        sig = (
            o.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo_.signature  # noqa: B950
        )
        sig = "".join(objc.splitSignature(sig))
        self.assertEqual(sig, "v@:@@@@@:i")

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSOpenPanel.resolvesAliases)
        self.assertArgIsBOOL(AppKit.NSOpenPanel.setResolvesAliases_, 0)
        self.assertResultIsBOOL(AppKit.NSOpenPanel.canChooseDirectories)
        self.assertArgIsBOOL(AppKit.NSOpenPanel.setCanChooseDirectories_, 0)
        self.assertResultIsBOOL(AppKit.NSOpenPanel.allowsMultipleSelection)
        self.assertArgIsBOOL(AppKit.NSOpenPanel.setAllowsMultipleSelection_, 0)
        self.assertResultIsBOOL(AppKit.NSOpenPanel.canChooseFiles)
        self.assertArgIsBOOL(AppKit.NSOpenPanel.setCanChooseFiles_, 0)

        self.assertArgIsSEL(
            AppKit.NSOpenPanel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo_,  # noqa: B950
            5,
            b"v@:@" + objc._C_NSInteger + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSOpenPanel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo_,  # noqa: B950
            6,
            b"^v",
        )

        self.assertArgIsSEL(
            AppKit.NSOpenPanel.beginForDirectory_file_types_modelessDelegate_didEndSelector_contextInfo_,  # noqa: B950
            4,
            b"v@:@" + objc._C_NSInteger + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSOpenPanel.beginForDirectory_file_types_modelessDelegate_didEndSelector_contextInfo_,  # noqa: B950
            5,
            b"^v",
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(AppKit.NSOpenPanel.canResolveUbiquitousConflicts)
        self.assertArgIsBOOL(AppKit.NSOpenPanel.setCanResolveUbiquitousConflicts_, 0)
        self.assertResultIsBOOL(AppKit.NSOpenPanel.canDownloadUbiquitousContents)
        self.assertArgIsBOOL(AppKit.NSOpenPanel.setCanDownloadUbiquitousContents_, 0)

    def test_issue_272(self):
        panel = AppKit.NSOpenPanel.openPanel()
        panel.setAllowedFileTypes_([".html", ".txt"])



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSMorphology(TestCase):
    def test_constants(self):
        self.assertEqual(Foundation.NSGrammaticalGenderNotSet, 0)
        self.assertEqual(Foundation.NSGrammaticalGenderFeminine, 1)
        self.assertEqual(Foundation.NSGrammaticalGenderMasculine, 2)
        self.assertEqual(Foundation.NSGrammaticalGenderNeuter, 3)

        self.assertEqual(Foundation.NSGrammaticalPartOfSpeechNotSet, 0)
        self.assertEqual(Foundation.NSGrammaticalPartOfSpeechDeterminer, 1)
        self.assertEqual(Foundation.NSGrammaticalPartOfSpeechPronoun, 2)
        self.assertEqual(Foundation.NSGrammaticalPartOfSpeechLetter, 3)
        self.assertEqual(Foundation.NSGrammaticalPartOfSpeechAdverb, 4)
        self.assertEqual(Foundation.NSGrammaticalPartOfSpeechParticle, 5)
        self.assertEqual(Foundation.NSGrammaticalPartOfSpeechAdjective, 6)
        self.assertEqual(Foundation.NSGrammaticalPartOfSpeechAdposition, 7)
        self.assertEqual(Foundation.NSGrammaticalPartOfSpeechVerb, 8)
        self.assertEqual(Foundation.NSGrammaticalPartOfSpeechNoun, 9)
        self.assertEqual(Foundation.NSGrammaticalPartOfSpeechConjunction, 10)
        self.assertEqual(Foundation.NSGrammaticalPartOfSpeechNumeral, 11)
        self.assertEqual(Foundation.NSGrammaticalPartOfSpeechInterjection, 12)
        self.assertEqual(Foundation.NSGrammaticalPartOfSpeechPreposition, 13)
        self.assertEqual(Foundation.NSGrammaticalPartOfSpeechAbbreviation, 14)

        self.assertEqual(Foundation.NSGrammaticalNumberNotSet, 0)
        self.assertEqual(Foundation.NSGrammaticalNumberSingular, 1)
        self.assertEqual(Foundation.NSGrammaticalNumberZero, 2)
        self.assertEqual(Foundation.NSGrammaticalNumberPlural, 3)
        self.assertEqual(Foundation.NSGrammaticalNumberPluralTwo, 4)
        self.assertEqual(Foundation.NSGrammaticalNumberPluralFew, 5)
        self.assertEqual(Foundation.NSGrammaticalNumberPluralMany, 6)

    @min_os_level("12.0")
    def test_methods(self):
        self.assertResultIsBOOL(
            Foundation.NSMorphology.setCustomPronoun_forLanguage_error_
        )
        self.assertArgIsOut(
            Foundation.NSMorphology.setCustomPronoun_forLanguage_error_, 2
        )

        self.assertResultIsBOOL(
            Foundation.NSMorphologyCustomPronoun.isSupportedForLanguage_
        )

        self.assertResultIsBOOL(Foundation.NSMorphology.isUnspecified)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSFileWrapper(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(
            AppKit.NSFileWrapper.writeToFile_atomically_updateFilenames_
        )
        self.assertArgIsBOOL(
            AppKit.NSFileWrapper.writeToFile_atomically_updateFilenames_, 1
        )
        self.assertArgIsBOOL(
            AppKit.NSFileWrapper.writeToFile_atomically_updateFilenames_, 2
        )

        self.assertResultIsBOOL(AppKit.NSFileWrapper.isRegularFile)
        self.assertResultIsBOOL(AppKit.NSFileWrapper.isDirectory)
        self.assertResultIsBOOL(AppKit.NSFileWrapper.isSymbolicLink)
        self.assertResultIsBOOL(AppKit.NSFileWrapper.needsToBeUpdatedFromPath_)
        self.assertResultIsBOOL(AppKit.NSFileWrapper.updateFromPath_)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(AppKit.NSFileWrapperReadingImmediate, 1 << 0)
        self.assertEqual(AppKit.NSFileWrapperReadingWithoutMapping, 1 << 1)
        self.assertEqual(AppKit.NSFileWrapperWritingAtomic, 1 << 0)
        self.assertEqual(AppKit.NSFileWrapperWritingWithNameUpdating, 1 << 1)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgIsOut(AppKit.NSFileWrapper.initWithURL_options_error_, 2)
        self.assertResultIsBOOL(AppKit.NSFileWrapper.matchesContentsOfURL_)
        self.assertResultIsBOOL(AppKit.NSFileWrapper.readFromURL_options_error_)
        self.assertArgIsOut(AppKit.NSFileWrapper.readFromURL_options_error_, 2)

        self.assertResultIsBOOL(
            AppKit.NSFileWrapper.writeToURL_options_originalContentsURL_error_
        )
        self.assertArgIsOut(
            AppKit.NSFileWrapper.writeToURL_options_originalContentsURL_error_, 3
        )



import Foundation
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSBackgroundActivityScheduler(TestCase):
    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(Foundation.NSBackgroundActivityResultFinished, 1)
        self.assertEqual(Foundation.NSBackgroundActivityResultDeferred, 2)

    @min_os_level("10.10")
    def testMethods10_10(self):
        Foundation.NSBackgroundActivityCompletionHandler = b"v" + objc._C_NSInteger

        self.assertResultIsBOOL(Foundation.NSBackgroundActivityScheduler.repeats)
        self.assertArgIsBOOL(Foundation.NSBackgroundActivityScheduler.setRepeats_, 0)
        self.assertArgIsBlock(
            Foundation.NSBackgroundActivityScheduler.scheduleWithBlock_,
            0,
            Foundation.NSBackgroundActivityCompletionHandler,
        )
        self.assertResultIsBOOL(Foundation.NSBackgroundActivityScheduler.shouldDefer)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSTextCheckingClientHelper(AppKit.NSObject):
    def annotatedSubstringForProposedRange_actualRange_(self, a, b):
        return 1

    def setAnnotations_range_(self, a, b):
        pass

    def addAnnotations_range_(self, a, b):
        pass

    def removeAnnotation_range_(self, a, b):
        pass

    def replaceCharactersInRange_withAnnotatedString_(self, a, b):
        pass

    def selectAndShowRange_(self, a):
        pass

    def viewForRange_firstRect_actualRange_(self, a, b, c):
        return 1

    def autocorrectionType(self):
        return 1

    def spellCheckingType(self):
        return 1

    def grammarCheckingType(self):
        return 1

    def smartQuotesType(self):
        return 1

    def smartDashesType(self):
        return 1

    def smartInsertDeleteType(self):
        return 1

    def textReplacementType(self):
        return 1

    def dataDetectionType(self):
        return 1

    def linkDetectionType(self):
        return 1

    def textCompletionType(self):
        return 1

    def setAutocorrectionType_(self, a):
        pass

    def setSpellCheckingType_(self, a):
        pass

    def setGrammarCheckingType_(self, a):
        pass

    def setSmartQuotesType_(self, a):
        pass

    def setSmartDashesType_(self, a):
        pass

    def setSmartInsertDeleteType_(self, a):
        pass

    def setTextReplacementType_(self, a):
        pass

    def setDataDetectionType_(self, a):
        pass

    def setLinkDetectionType_(self, a):
        pass

    def setTextCompletionType_(self, a):
        pass


class TestNSTextCheckingClient(TestCase):
    def test_constants(self):
        self.assertEqual(AppKit.NSTextInputTraitTypeDefault, 0)
        self.assertEqual(AppKit.NSTextInputTraitTypeNo, 1)
        self.assertEqual(AppKit.NSTextInputTraitTypeYes, 2)

    def test_methods(self):
        self.assertResultHasType(
            TestNSTextCheckingClientHelper.autocorrectionType, objc._C_NSInteger
        )
        self.assertResultHasType(
            TestNSTextCheckingClientHelper.spellCheckingType, objc._C_NSInteger
        )
        self.assertResultHasType(
            TestNSTextCheckingClientHelper.grammarCheckingType, objc._C_NSInteger
        )
        self.assertResultHasType(
            TestNSTextCheckingClientHelper.smartQuotesType, objc._C_NSInteger
        )
        self.assertResultHasType(
            TestNSTextCheckingClientHelper.smartDashesType, objc._C_NSInteger
        )
        self.assertResultHasType(
            TestNSTextCheckingClientHelper.smartInsertDeleteType, objc._C_NSInteger
        )
        self.assertResultHasType(
            TestNSTextCheckingClientHelper.textReplacementType, objc._C_NSInteger
        )
        self.assertResultHasType(
            TestNSTextCheckingClientHelper.dataDetectionType, objc._C_NSInteger
        )
        self.assertResultHasType(
            TestNSTextCheckingClientHelper.linkDetectionType, objc._C_NSInteger
        )
        self.assertResultHasType(
            TestNSTextCheckingClientHelper.textCompletionType, objc._C_NSInteger
        )

        self.assertArgHasType(
            TestNSTextCheckingClientHelper.setAutocorrectionType_, 0, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.setSpellCheckingType_, 0, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.setGrammarCheckingType_, 0, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.setSmartQuotesType_, 0, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.setSmartDashesType_, 0, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.setSmartInsertDeleteType_,
            0,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.setTextReplacementType_, 0, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.setDataDetectionType_, 0, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.setLinkDetectionType_, 0, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.setTextCompletionType_, 0, objc._C_NSInteger
        )

        self.assertArgHasType(
            TestNSTextCheckingClientHelper.annotatedSubstringForProposedRange_actualRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.annotatedSubstringForProposedRange_actualRange_,
            1,
            b"o^" + AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.setAnnotations_range_,
            1,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.addAnnotations_range_,
            1,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.removeAnnotation_range_,
            1,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.replaceCharactersInRange_withAnnotatedString_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.selectAndShowRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.viewForRange_firstRect_actualRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.viewForRange_firstRect_actualRange_,
            1,
            b"o^" + AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextCheckingClientHelper.viewForRange_firstRect_actualRange_,
            2,
            b"o^" + AppKit.NSRange.__typestr__,
        )

    @min_sdk_level("10.15")
    def test_protocols(self):
        objc.protocolNamed("NSTextInputTraits")
        objc.protocolNamed("NSTextCheckingClient")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level

NSStoryboardControllerCreator = b"@@"


class TestNSStoryboard(TestCase):
    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertArgIsBlock(
            AppKit.NSStoryboard.instantiateInitialControllerWithCreator_,
            0,
            NSStoryboardControllerCreator,
        )
        self.assertArgIsBlock(
            AppKit.NSStoryboard.instantiateControllerWithIdentifier_creator_,
            1,
            NSStoryboardControllerCreator,
        )



from AppKit import NSFormatter, NSString
from PyObjCTools import AppHelper

try:
    unicode
except NameError:
    unicode = str


class MyFormatter(NSFormatter):
    def stringForObjectValue_(self, product):
        if isinstance(product, (str, unicode)):
            return product
        return str(product)

    def getObjectValue_forString_errorDescription_(self, value, upc, error):
        print(self, upc)

        if not upc:
            print("No data")
            return True, None, None

        print("Have data")
        return False, None, NSString.stringWithString_("Foo the {}".format("bar"))


AppHelper.runEventLoop()



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSTabViewHelper(AppKit.NSObject):
    def tabView_shouldSelectTabViewItem_(self, tv, it):
        return 1


class TestNSTabView(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSTopTabsBezelBorder, 0)
        self.assertEqual(AppKit.NSLeftTabsBezelBorder, 1)
        self.assertEqual(AppKit.NSBottomTabsBezelBorder, 2)
        self.assertEqual(AppKit.NSRightTabsBezelBorder, 3)
        self.assertEqual(AppKit.NSNoTabsBezelBorder, 4)
        self.assertEqual(AppKit.NSNoTabsLineBorder, 5)
        self.assertEqual(AppKit.NSNoTabsNoBorder, 6)

        self.assertEqual(AppKit.NSTabPositionNone, 0)
        self.assertEqual(AppKit.NSTabPositionTop, 1)
        self.assertEqual(AppKit.NSTabPositionLeft, 2)
        self.assertEqual(AppKit.NSTabPositionBottom, 3)
        self.assertEqual(AppKit.NSTabPositionRight, 4)

        self.assertEqual(AppKit.NSTabViewBorderTypeNone, 0)
        self.assertEqual(AppKit.NSTabViewBorderTypeLine, 1)
        self.assertEqual(AppKit.NSTabViewBorderTypeBezel, 2)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSTabView.allowsTruncatedLabels)
        self.assertResultIsBOOL(AppKit.NSTabView.drawsBackground)
        self.assertArgIsBOOL(AppKit.NSTabView.setAllowsTruncatedLabels_, 0)
        self.assertArgIsBOOL(AppKit.NSTabView.setDrawsBackground_, 0)

    def testProtocols(self):
        self.assertResultIsBOOL(TestNSTabViewHelper.tabView_shouldSelectTabViewItem_)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSPasteboardHelper(AppKit.NSObject):
    def writingOptionsForType_pasteboard_(self, t, p):
        return 1

    def readingOptionsForType_pasteboard_(self, t, p):
        return 1


class TestNSPasteboard(TestCase):
    def testConstants(self):
        self.assertIsInstance(AppKit.NSStringPboardType, str)
        self.assertIsInstance(AppKit.NSFilenamesPboardType, str)
        self.assertIsInstance(AppKit.NSPostScriptPboardType, str)
        self.assertIsInstance(AppKit.NSTIFFPboardType, str)
        self.assertIsInstance(AppKit.NSRTFPboardType, str)
        self.assertIsInstance(AppKit.NSTabularTextPboardType, str)
        self.assertIsInstance(AppKit.NSFontPboardType, str)
        self.assertIsInstance(AppKit.NSRulerPboardType, str)
        self.assertIsInstance(AppKit.NSFileContentsPboardType, str)
        self.assertIsInstance(AppKit.NSColorPboardType, str)
        self.assertIsInstance(AppKit.NSRTFDPboardType, str)
        self.assertIsInstance(AppKit.NSHTMLPboardType, str)
        self.assertIsInstance(AppKit.NSPICTPboardType, str)
        self.assertIsInstance(AppKit.NSURLPboardType, str)
        self.assertIsInstance(AppKit.NSPDFPboardType, str)
        self.assertIsInstance(AppKit.NSVCardPboardType, str)
        self.assertIsInstance(AppKit.NSFilesPromisePboardType, str)
        self.assertIsInstance(AppKit.NSInkTextPboardType, str)
        self.assertIsInstance(AppKit.NSGeneralPboard, str)
        self.assertIsInstance(AppKit.NSFontPboard, str)
        self.assertIsInstance(AppKit.NSRulerPboard, str)
        self.assertIsInstance(AppKit.NSFindPboard, str)
        self.assertIsInstance(AppKit.NSDragPboard, str)

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertIsInstance(AppKit.NSMultipleTextSelectionPboardType, str)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(AppKit.NSPasteboardTypeTextFinderOptions, str)

    def testFunctions(self):
        tp = v = AppKit.NSCreateFilenamePboardType("test/jpeg")
        self.assertIsInstance(v, str)

        v = AppKit.NSCreateFileContentsPboardType("test/jpeg")
        self.assertIsInstance(v, str)

        v = AppKit.NSGetFileType(tp)
        self.assertIsInstance(v, str)

        v = AppKit.NSGetFileTypes([tp])
        self.assertIsInstance(v, AppKit.NSArray)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSPasteboard.setData_forType_)
        self.assertResultIsBOOL(AppKit.NSPasteboard.setPropertyList_forType_)
        self.assertResultIsBOOL(AppKit.NSPasteboard.setString_forType_)
        self.assertResultIsBOOL(AppKit.NSPasteboard.writeFileContents_)
        self.assertResultIsBOOL(AppKit.NSPasteboard.writeFileWrapper_)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(AppKit.NSPasteboardTypeString, str)
        self.assertIsInstance(AppKit.NSPasteboardTypePDF, str)
        self.assertIsInstance(AppKit.NSPasteboardTypeTIFF, str)
        self.assertIsInstance(AppKit.NSPasteboardTypePNG, str)
        self.assertIsInstance(AppKit.NSPasteboardTypeRTF, str)
        self.assertIsInstance(AppKit.NSPasteboardTypeRTFD, str)
        self.assertIsInstance(AppKit.NSPasteboardTypeHTML, str)
        self.assertIsInstance(AppKit.NSPasteboardTypeTabularText, str)
        self.assertIsInstance(AppKit.NSPasteboardTypeFont, str)
        self.assertIsInstance(AppKit.NSPasteboardTypeRuler, str)
        self.assertIsInstance(AppKit.NSPasteboardTypeColor, str)
        self.assertIsInstance(AppKit.NSPasteboardTypeSound, str)
        self.assertIsInstance(AppKit.NSPasteboardTypeMultipleTextSelection, str)
        self.assertIsInstance(AppKit.NSPasteboardTypeFindPanelSearchOptions, str)

        self.assertIsInstance(AppKit.NSPasteboardURLReadingFileURLsOnlyKey, str)
        self.assertIsInstance(
            AppKit.NSPasteboardURLReadingContentsConformToTypesKey, str
        )

        self.assertEqual(AppKit.NSPasteboardWritingPromised, 1 << 9)

        self.assertEqual(AppKit.NSPasteboardReadingAsData, 0)
        self.assertEqual(AppKit.NSPasteboardReadingAsString, 1 << 0)
        self.assertEqual(AppKit.NSPasteboardReadingAsPropertyList, 1 << 1)
        self.assertEqual(AppKit.NSPasteboardReadingAsKeyedArchive, 1 << 2)

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertEqual(AppKit.NSPasteboardContentsCurrentHostOnly, 1 << 0)

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(AppKit.NSPasteboardNameGeneral, str)
        self.assertIsInstance(AppKit.NSPasteboardNameFont, str)
        self.assertIsInstance(AppKit.NSPasteboardNameRuler, str)
        self.assertIsInstance(AppKit.NSPasteboardNameFind, str)
        self.assertIsInstance(AppKit.NSPasteboardNameDrag, str)
        self.assertIsInstance(AppKit.NSPasteboardTypeURL, str)
        self.assertIsInstance(AppKit.NSPasteboardTypeFileURL, str)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(AppKit.NSPasteboard.writeObjects_)
        self.assertResultIsBOOL(
            AppKit.NSPasteboard.canReadItemWithDataConformingToTypes_
        )
        self.assertResultIsBOOL(AppKit.NSPasteboard.canReadObjectForClasses_options_)

        self.assertResultIsBOOL(AppKit.NSPasteboard.setPropertyList_forType_)
        self.assertResultIsBOOL(AppKit.NSPasteboard.setString_forType_)

    @min_sdk_level("10.6")
    def testProtocolObjects(self):
        objc.protocolNamed("NSPasteboardWriting")
        objc.protocolNamed("NSPasteboardReading")

    @min_sdk_level("10.14")
    def testProtocolObjects10_14(self):
        objc.protocolNamed("NSPasteboardTypeOwner")

    def testProtocols(self):
        self.assertResultHasType(
            TestNSPasteboardHelper.writingOptionsForType_pasteboard_, objc._C_NSUInteger
        )

        self.assertResultHasType(
            TestNSPasteboardHelper.readingOptionsForType_pasteboard_, objc._C_NSUInteger
        )
